@@ -3,11 +3,11 @@
 ;; Copyright (C) 2025 Jordan Mandel
 ;; Author: Jordan Mandel <jordan.mandel@live.com>
 ;; Created: 2025-04-22
-;; Package-Version: 1.0.0
-;; Package-Revision: v1.0.0-0-ga2e240e8ffdf
+;; Package-Version: 20251031.2050
+;; Package-Revision: 5f61e718133c
 ;; Package-Requires: ((emacs "28.1") (evil "1.0.8") (god-mode "2.12.0"))
 ;; Keywords: convenience, emulation, evil, god-mode
-;; URL: https://github.com/jam1015/evil-god-toggle
+;; Homepage: https://github.com/jam1015/evil-god-toggle
 ;;
 ;; This file is not part of GNU Emacs.
 ;;
@@ -35,8 +35,7 @@
 ;; Provided states:
 ;; - `god': Full persistent God mode.
 ;; - `god-once': Enter God mode for one command, then return to Evil.
-;; - `god-off': Explicitly turns God mode off persistently.
-;;
+;; - `god-off': Like evil's built-in Emacs state.  Easy to switch to and from.
 ;; Ideal for users blending Emacs and Vim-style modal editing.
 ;;
 ;;; Code:
@@ -59,7 +58,6 @@
 
 
 ;; customization options
-
 ;;;###autoload
 (defcustom evil-god-toggle-persist-visual 'always
   "Control persistence of visual selection when toggling."
@@ -69,33 +67,41 @@
                  (const :tag "Never" nil))
   :group 'evil-god-toggle)
 
+
+
+;; customization options
 ;;;###autoload
-(defcustom evil-god-toggle-global nil
-  "If non-nil, use `god-mode-all` instead of `god-local-mode` (per-buffer)."
-  :type 'boolean
+(defcustom evil-god-toggle-persist-visual-once 'follow
+  "Control persistence of visual selection when toggling."
+  :type '(choice (const :tag "Always" always)
+                 (const :tag "To God" to-god)
+                 (const :tag "To Evil" to-evil)
+                 (const :tag "Follow" follow)
+                 (const :tag "Never" nil))
   :group 'evil-god-toggle)
 
 
-(defvar evil-god-toggle--once-buffer nil
-  "The buffer in which `god-once` was entered, for restoring Evil state.")
+;;;###autoload
+(defcustom evil-god-toggle-target-state-alist nil
+  "Alist mapping previous Evil states to target states when exiting god-once.
+Each entry is (PREVIOUS-STATE . TARGET-STATE).
+If a previous state is not found in this alist, it will transition
+to that state directly.
+
+Example: '((insert . normal) (replace . normal))
+This would transition to normal state when exiting god-once from
+insert or replace states."
+  :type '(alist :key-type symbol :value-type symbol)
+  :group 'evil-god-toggle)
 
 (defvar evil-god-toggle-mode-map (make-sparse-keymap)
   "Keymap for `evil-god-toggle-mode'.")
 
 
-(defvar-local evil-god-toggle--visual-hooks-removed nil
-  "Non-nil if evil-god-toggle removed visual mark hooks in this buffer.")
-
-(defvar-local evil-god-toggle--has-fix-last-command-hook nil
-  "Non-nil if `evil-god-toggle--fix-last-command` in local `pre-command-hook`.")
-
-(defvar-local evil-god-toggle--has-exit-once-hook nil
-  "Non-nil if `evil-god-toggle--exit-once` in local `post-command-hook`.")
-
 (defvar evil-god-toggle--last-command nil
   "Command executed just before entering god state.")
 
-(defvar evil-god-toggle--visual-forward nil
+(defvar-local evil-god-toggle--visual-forward nil
   "Non-nil if the saved region was selected forward (mark ≤ point).")
 (defvar-local evil-god-toggle--visual-beg nil
   "Buffer position of region start to restore when toggling back to Evil visual.")
@@ -118,6 +124,76 @@ Creates `god-mode' states for Evil."
     (remove-hook 'evil-visual-state-entry-hook
                  #'evil-god-toggle--check-and-update-previous-state-visual)))
 
+
+;;;; god-state hooks ;;;;;;;;;;
+(evil-define-state god
+  "God state." :tag " <G> "
+  :message "-- GOD MODE --"
+  :entry-hook  evil-god-toggle--start-hook-fun
+  :exit-hook   evil-god-toggle--stop-hook-fun
+  :input-method t :intercept-esc nil)
+
+(defun evil-god-toggle--start-hook-fun ()
+  "Run before entering `evil-god-state'."
+    (evil-god-toggle--remove-visual-hooks)
+  ;; either global or local God
+  (evil-god-toggle--enable-god))
+
+(defun evil-god-toggle--stop-hook-fun ()
+  "Run before exiting `evil-god-state'."
+  ;; restore visual hooks, then turn off God
+  (evil-god-toggle--remove-transient-hooks)
+  (evil-god-toggle--restore-visual-hooks)
+  (evil-god-toggle--disable-god)
+  (evil-normalize-keymaps))
+
+;; god once and hooks ;;;;;;;;;;;;;;;;;;;;;;;;
+
+(evil-define-state god-once
+  "God state (once)." :tag " <G (once)> "
+  :message "-- GOD MODE (once) --"
+  :entry-hook  evil-god-toggle--once-start-hook-fun
+  :exit-hook   evil-god-toggle--once-stop-hook-fun
+  :input-method t :intercept-esc nil)
+
+(defun evil-god-toggle--once-start-hook-fun ()
+  "Run before entering `evil-god-once-state'."
+ (evil-god-toggle--remove-visual-hooks)
+  ;; either global or local God
+  (evil-god-toggle--add-add-exit-once)
+  (evil-god-toggle--enable-god))
+
+(defun evil-god-toggle--once-stop-hook-fun ()
+  "Run before exiting `evil-god-once-state'."
+  ;; restore visual hooks, then turn off God
+  (evil-god-toggle--restore-visual-hooks)
+  (evil-god-toggle--disable-god)
+  (evil-god-toggle--remove-transient-hooks)
+  (evil-normalize-keymaps))
+
+;; god off and hooks ;;;;;;;;;;;;;;;;;;;;;;;;
+
+(evil-define-state god-off
+  "God state (off)." :tag " <G (off)> "
+  :message "-- GOD MODE (OFF) --"
+  :entry-hook  evil-god-toggle--off-start-hook-fun
+  :exit-hook   evil-god-toggle--off-stop-hook-fun
+  :input-method t :intercept-esc nil)
+
+(defun evil-god-toggle--off-start-hook-fun ()
+  "Run before entering `evil-god-off-state'."
+  (evil-god-toggle--remove-visual-hooks)
+  ;; either global or local God
+  (evil-god-toggle--disable-god))
+
+(defun evil-god-toggle--off-stop-hook-fun ()
+  "Run before exiting `evil-god-off-state'."
+  ;; same cleanup for God-off exit
+  (evil-god-toggle--restore-visual-hooks))
+
+
+
+
 ;;;###autoload
 (defun evil-god-toggle-god-toggle ()
   "Toggle between God mode and Evil mode.
@@ -133,30 +209,46 @@ Optional helper function."
      (t (evil-god-toggle-execute-in-god-state)))))
 
 ;;;###autoload
-(defun evil-god-toggle-execute-in-god-off-state ()
-  "Exit God mode (force `god-off'), optionally restoring visual selection.
-If called from the minibuffer, signal a user-error."
-  (interactive)
-  (if (minibufferp)
-      (user-error "Cannot enter `god-off' mode from minibuffer")
-    ;; original behavior
-    (evil-god-toggle--add-transient-hooks t nil)
-    (setq evil-god-toggle--last-command last-command)
-    (evil-god-toggle--maybe-restore-visual 'evil-god-off-state)))
+(defun evil-god-toggle-once (&optional move-forward)
+  "Enter God mode for exactly one command, then return to Evil.
+If MOVE-FORWARD is non-nil, move cursor one character forward after entering.
+If already in `god-once', signal a user-error."
+  (interactive "P")
+  (cond
+   ((minibufferp)
+    (user-error "Cannot enter god-once mode from minibuffer"))
+   ((eq evil-state 'god-once)
+    (user-error "Already in god-once state"))
+   ((eq evil-state 'god)
+    (user-error "Already in god state"))
+   (t (evil-god-toggle--execute-in-god-once-state move-forward))))
+
 
 ;;;###autoload
-(defun evil-god-toggle-execute-in-god-state ()
-  "Go into God state, preserving visual selection if configured."
-  (interactive)
+(defun evil-god-toggle-execute-in-god-state (&optional move-forward)
+  "Go into God state, preserving visual selection if configured.
+If MOVE-FORWARD is non-nil, move cursor one character forward after entering."
+  (interactive "P")
   (setq evil-god-toggle--last-command last-command)
-  (evil-god-toggle--add-transient-hooks t nil)
-  (evil-god-toggle--maybe-restore-visual  'evil-god-state))
+  (evil-god-toggle--add-fix-last)
+  (evil-god-toggle--switch-state-maybe-restore-region 'evil-god-state)
+  (when move-forward
+    (forward-char 1)))
+
+;;;###autoload
+(defun evil-god-toggle-execute-in-god-state-forward ()
+  "Enter God state and move forward one character."
+  (interactive)
+  (evil-god-toggle-execute-in-god-state t))
+
+
 
 ;;;###autoload
 (defun evil-god-toggle-stop-execute-in-god-state (target)
   "Wrapper for leaving god state and switching to TARGET evil state.
 TARGET should be a symbol: `normal', `insert', or `visual'.
-Does not respect `evil-god-toggle-visual-persist'"
+Does not respect `evil-god-toggle-persist-visual'
+`evil-god-toggle-stop-god-state-maybe-visual' is smarter"
   (interactive)
   (pcase target
     ('normal (evil-god-toggle--transition-to-normal))
@@ -165,15 +257,45 @@ Does not respect `evil-god-toggle-visual-persist'"
     (_       (evil-god-toggle--transition-to-normal))) ;; fallback
   (force-mode-line-update))
 
+(defun evil-god-toggle--execute-in-god-once-state (&optional move-forward)
+  "Go into God-once state, preserving visual selection if configured.
+If MOVE-FORWARD is non-nil, move cursor one character forward after entering."
+  (interactive "P")
+  (setq evil-god-toggle--last-command last-command)
+  (evil-god-toggle--add-fix-last)
+  (evil-god-toggle--switch-state-maybe-restore-region-once 'evil-god-once-state)
+  (when move-forward
+    (forward-char 1)))
+
 ;;;###autoload
-(defun evil-god-toggle-stop-choose-state (alternate_target)
+(defun evil-god-toggle-once-forward ()
+  "Enter God-once state and move forward one character."
+  (interactive)
+  (evil-god-toggle-once t))
+
+
+;;;###autoload
+(defun evil-god-toggle-execute-in-god-off-state ()
+  "Exit God mode (force `god-off'), optionally restoring visual selection.
+If called from the minibuffer, signal a user-error."
+  (interactive)
+  (if (minibufferp)
+      (user-error "Cannot enter `god-off' mode from minibuffer")
+    ;; original behavior
+    (evil-god-toggle--add-fix-last)
+    (setq evil-god-toggle--last-command last-command)
+    (evil-god-toggle--switch-state-maybe-restore-region 'evil-god-off-state)))
+
+
+;;;###autoload
+(defun evil-god-toggle-stop-god-state-maybe-visual (alternate-target)
   "From God, toggle back into Evil, choosing appropriate state.
-Restore visual or going to state specified by `ALTERNATE_TARGET'.
-Alternate_target can be \='normal \='insert or \='visual .
-If there's an active region AND either persist-visual flag is t,
+Restore visual or going to state specified by `ALTERNATE-TARGET'.
+alternate-target can be `normal', `insert', or `visual'.
+If there's an active region AND either persist-visual flag is active,
 stash its bounds **and** direction, then call visual; else normal.
 This is the function that should be used for keybindings because
-it it respects `evil-god-toggle-visual-persist'"
+it it respects `evil-god-toggle-persist-visual'"
   (interactive)
   ;; only if a region is selected
   (if (and (use-region-p)  (memq evil-god-toggle-persist-visual '(always to-evil)))
@@ -188,21 +310,36 @@ it it respects `evil-god-toggle-visual-persist'"
               evil-god-toggle--visual-end     end
               evil-god-toggle--visual-forward forward)
 	(evil-god-toggle-stop-execute-in-god-state 'visual))
-    (evil-god-toggle-stop-execute-in-god-state alternate_target)))
+    (evil-god-toggle-stop-execute-in-god-state alternate-target)))
 
-;;;###autoload
-(defun evil-god-toggle-once ()
-  "Enter God mode for exactly one command, then return to Evil.
-If already in `god-once', signal a user-error."
+
+(defun evil-god-toggle-stop-god-state-maybe-visual-once (alternate-target)
+  "From God-once, toggle back into Evil, choosing appropriate state.
+Restore visual or going to state specified by `ALTERNATE-TARGET'.
+alternate-target can be `normal', `insert', or `visual'.
+If there's an active region AND either persist-visual flag active,
+stash its bounds **and** direction, then call visual; else normal.
+This is the function that should be used for keybindings because
+it it respects `evil-god-toggle-persist-visual'"
   (interactive)
-  (cond
-   ((minibufferp)
-    (user-error "Cannot enter god-once mode from minibuffer"))
-   ((eq evil-state 'god-once)
-    (user-error "Already in god-once state"))
-   ((eq evil-state 'god)
-    (user-error "Already in god state"))
-   (t (evil-god-once-state))))
+  ;; only if a region is selected
+  (let* ((persist-setting (if (eq evil-god-toggle-persist-visual-once 'follow)
+                              evil-god-toggle-persist-visual
+                              evil-god-toggle-persist-visual-once)))
+    
+    (if (and (use-region-p) (memq persist-setting '(always to-evil)))
+        (let* ((m       (mark))
+               (p       (point))
+               (beg     (min m p))
+               ;; Emacs's region-end is exclusive, so -1 to make it inclusive
+               (end     (1- (max m p)))
+               ;; mark before point?
+               (forward (<= m p)))
+          (setq evil-god-toggle--visual-beg     beg
+                evil-god-toggle--visual-end     end
+                evil-god-toggle--visual-forward forward)
+          (evil-god-toggle-stop-execute-in-god-state 'visual))
+      (evil-god-toggle-stop-execute-in-god-state alternate-target))))
 
 ;;;###autoload
 (defun evil-god-toggle-bail ()
@@ -212,161 +349,98 @@ If already in `god-once', signal a user-error."
   (evil-god-toggle--stop-hook-fun)
   (evil-god-toggle-stop-execute-in-god-state 'normal))
 
-(evil-define-state god
-  "God state." :tag " <G> "
-  :message "-- GOD MODE --"
-  :entry-hook  evil-god-toggle--start-hook-fun
-  :exit-hook   evil-god-toggle--stop-hook-fun
-  :input-method t :intercept-esc nil)
-
-(evil-define-state god-once
-  "God state (once)." :tag " <G (once)> "
-  :message "-- GOD MODE (once) --"
-  :entry-hook  evil-god-toggle--once-start-hook-fun
-  :exit-hook   evil-god-toggle--once-stop-hook-fun
-  :input-method t :intercept-esc nil)
-
-(evil-define-state god-off
-  "God state (off)." :tag " <G (off)> "
-  :message "-- GOD MODE (OFF) --"
-  :entry-hook  evil-god-toggle--off-start-hook-fun
-  :exit-hook   evil-god-toggle--off-stop-hook-fun
-  :input-method t :intercept-esc nil)
-
-
-
-
 (defun evil-god-toggle--remove-transient-hooks ()
   "Remove God mode transient hooks from all buffers that added them."
   (dolist (buf (buffer-list))
     (with-current-buffer buf
-      (when evil-god-toggle--has-fix-last-command-hook
         (remove-hook 'pre-command-hook #'evil-god-toggle--fix-last-command t)
-        (setq-local evil-god-toggle--has-fix-last-command-hook nil))
-      (when evil-god-toggle--has-exit-once-hook
         (remove-hook 'post-command-hook #'evil-god-toggle--exit-once t)
-        (setq-local evil-god-toggle--has-exit-once-hook nil)))))
+        (remove-hook 'post-command-hook #'evil-god-toggle--add-exit-once t))))
 
-(defun evil-god-toggle--add-transient-hooks (add-fix-last add-exit-once)
-  "Add transient God-mode hooks locally to current buffer.
-ADD-FIX-LAST:
-  non-nil adds `evil-god-toggle--fix-last-command` to `pre-command-hook`.
-ADD-EXIT-ONCE:
-  non-nil adds `evil-god-toggle--exit-once` to `post-command-hook`."
-  (when (and add-fix-last (not evil-god-toggle--has-fix-last-command-hook))
-    (add-hook 'pre-command-hook #'evil-god-toggle--fix-last-command nil t)
-    (setq-local evil-god-toggle--has-fix-last-command-hook t))
-  (when (and add-exit-once (not evil-god-toggle--has-exit-once-hook))
-    (add-hook 'post-command-hook #'evil-god-toggle--exit-once nil t)
-    (setq-local evil-god-toggle--has-exit-once-hook t)))
+(defun evil-god-toggle--add-fix-last ()
+  "Add `evil-god-toggle--fix-last-command` to `pre-command-hook`."
+  ;; add-hook won't add duplicates, so this is safe
+  (add-hook 'pre-command-hook #'evil-god-toggle--fix-last-command nil t))
+
+(defun evil-god-toggle--add-exit-once ()
+  "Add `evil-god-toggle--exit-once` to `post-command-hook`."
+  (add-hook 'post-command-hook #'evil-god-toggle--exit-once nil t))
+
+(defun evil-god-toggle--add-add-exit-once ()
+  "Add hook that adds `evil-god-toggle--exit-once` to `post-command-hook`."
+  (add-hook 'post-command-hook #'evil-god-toggle--add-exit-once nil t))
 
 (defun evil-god-toggle--check-and-update-previous-state-visual ()
-  "Set the previous Evil state to `normal' if it was `god' upon entering `visual'.
-When Evil transitions into `visual' state from `god', it may
-incorrectly keep `god' as the previous state.  This function checks
-if `evil-previous-state' is `god', resets it to `normal', and updates
-`evil-previous-state-alist' accordingly.  Put this on
-`evil-visual-state-entry-hook' to ensure Evil retains the correct
-previous state."
-  (when (eq evil-previous-state 'god)
-    (setq evil-previous-state 'normal)
-    (setq evil-previous-state-alist
-          (assq-delete-all 'god evil-previous-state-alist))
-    (add-to-list 'evil-previous-state-alist (cons 'god 'normal))))
+  "Set the previous Evil state to `normal' if it was `god' or `god-once' upon entering `visual'.
+When Evil transitions into `visual' state from `god' or `god-once', it may
+incorrectly keep those as the previous state. This function resets it to `normal'
+so that exiting visual returns to normal instead of a god state."
+  (when (memq evil-previous-state '(god god-once))
+    (setq evil-previous-state 'normal)))
 
 
 (defun evil-god-toggle--restore-visual-hooks ()
   "Re-add Evil visual activate/deactivate hooks where they were previously removed."
   (dolist (buf (buffer-list))
     (with-current-buffer buf
-      (when (and
-             (bound-and-true-p evil-local-mode)
-             (bound-and-true-p evil-god-toggle--visual-hooks-removed))
-        (evil-god-toggle--add-visual-hooks)
-        (setq-local evil-god-toggle--visual-hooks-removed nil)))))
+      ;; Only restore if evil-local-mode is active and hooks are missing
+      (when (and (bound-and-true-p evil-local-mode)
+                 (not (memq #'evil-visual-activate-hook activate-mark-hook)))
+        (evil-god-toggle--add-visual-hooks)))))
 
 (defun evil-god-toggle--disable-god ()
-  "Disable God mode, globally or buffer-locally per `evil-god-toggle-global'."
-  (if evil-god-toggle-global
-      (god-mode-all -1)
-    (god-local-mode -1)))
+  "Disable God mode."
+  (god-local-mode -1))
 
 (defun evil-god-toggle--enable-god ()
-  "Enable God mode, either globally or buffer-locally per `evil-god-toggle-global'."
-  (if evil-god-toggle-global
-      (god-mode-all 1)
-    (god-local-mode 1)))
-
-(defun evil-god-toggle--start-hook-fun ()
-  "Run before entering `evil-god-state'."
-  (when (or (member #'evil-visual-activate-hook activate-mark-hook)
-          (member #'evil-visual-deactivate-hook deactivate-mark-hook))
-    (evil-god-toggle--remove-visual-hooks))
-  ;; either global or local God
-  (evil-god-toggle--enable-god))
-
-(defun evil-god-toggle--stop-hook-fun ()
-  "Run before exiting `evil-god-state'."
-  ;; restore visual hooks, then turn off God
-  (evil-god-toggle--remove-transient-hooks)
-  (evil-god-toggle--restore-visual-hooks)
-  (evil-god-toggle--disable-god))
-
+  "Enable God mode."
+    (god-local-mode 1))
 
 (defun evil-god-toggle--fix-last-command ()
 "Internal: Restore `last-command` captured before entering God state."
         (setq last-command evil-god-toggle--last-command)
-        (remove-hook 'pre-command-hook #'evil-god-toggle--fix-last-command))
+        (remove-hook 'pre-command-hook #'evil-god-toggle--fix-last-command t))
 
-(defun evil-god-toggle--once-start-hook-fun ()
-  "Run before entering `evil-god-once-state'."
- (setq evil-god-toggle--last-command last-command
-       evil-god-toggle--once-buffer (current-buffer))
- (evil-god-toggle--remove-visual-hooks)
-  ;; either global or local God
-  (evil-god-toggle--add-transient-hooks t t)
-  (evil-god-toggle--enable-god))
+(defun evil-god-toggle--switch-state-maybe-restore-region-once (next-state-fn)
+  "Switch to NEXT-STATE-FN and optionally restore region if in visual mode.
+Skip restoration if in visual block mode since block selections have
+different semantics.  Note that this is for going to god."
+  (let* ((persist-setting (if (eq evil-god-toggle-persist-visual-once 'follow)
+                              evil-god-toggle-persist-visual
+                            evil-god-toggle-persist-visual-once)))
+    (if (and (evil-visual-state-p)
+             (not (eq evil-visual-selection 'block))  ; Skip for visual block
+             (memq persist-setting '(always to-god)))
+	(let ((mrk (mark))
+              (pnt (point)))
+          (funcall next-state-fn)
+          (set-mark mrk)
+          (goto-char pnt))
+      (funcall next-state-fn))
+    )
+  )
 
-(defun evil-god-toggle--once-stop-hook-fun ()
-  "Run before exiting `evil-god-once-state'."
-  ;; restore visual hooks, then turn off God
-  (evil-god-toggle--remove-transient-hooks)
-  (evil-god-toggle--restore-visual-hooks)
-  (evil-god-toggle--disable-god))
 
-(defun evil-god-toggle--off-start-hook-fun ()
-  "Run before entering `evil-god-off-state'."
-  (evil-god-toggle--remove-visual-hooks)
-  ;; either global or local God
-  (evil-god-toggle--disable-god))
+(defun evil-god-toggle--switch-state-maybe-restore-region (next-state-fn)
+  "Switch to NEXT-STATE-FN and optionally restore region if in visual mode.
+Skip restoration if in visual block mode since block selections have
+different semantics."
 
-(defun evil-god-toggle--off-stop-hook-fun ()
-  "Run before exiting `evil-god-off-state'."
-  ;; same cleanup for God-off exit
-  (evil-god-toggle--restore-visual-hooks)
-  (evil-god-toggle--disable-god))
-
-(defun evil-god-toggle--maybe-restore-visual (next-state-fn)
-  "Switch to NEXT-STATE-FN and optionally restore region if in visual mode."
   (if (and (evil-visual-state-p)
+           (not (eq evil-visual-selection 'block))  ; Skip for visual block
            (memq evil-god-toggle-persist-visual '(always to-god)))
       (let ((mrk (mark))
             (pnt (point)))
-
         (funcall next-state-fn)
         (set-mark mrk)
         (goto-char pnt))
-    ( funcall next-state-fn)))
+    (funcall next-state-fn))
+  )
 
 (defun evil-god-toggle--remove-visual-hooks ()
-  "Remove Evil's visual activate/deactivate hooks in current buffer.
-Prevents Evil's visual selection hooks from firing while in God mode.
-`evil-god-toggle--visual-hooks-removed' set to non-nil so we know to restore."
-  (when (or (member #'evil-visual-activate-hook activate-mark-hook)
-          (member #'evil-visual-deactivate-hook deactivate-mark-hook))
+  "Remove Evil's visual activate/deactivate hooks in current buffer."
   (remove-hook 'activate-mark-hook #'evil-visual-activate-hook t)
-  (remove-hook 'deactivate-mark-hook #'evil-visual-deactivate-hook t)
-  (setq-local evil-god-toggle--visual-hooks-removed t)))
+  (remove-hook 'deactivate-mark-hook #'evil-visual-deactivate-hook t))
 
 
 (defun evil-god-toggle--add-visual-hooks ()
@@ -391,6 +465,18 @@ Restores visual selection behavior by adding `evil-visual-activate-hook' to
 
 (defun evil-god-toggle--transition-to-visual ()
   "Enter Evil visual (charwise), restoring any stashed region & orientation."
+(if (bound-and-true-p rectangle-mark-mode)
+      (progn
+        (setq evil-god-toggle--visual-beg nil
+         evil-god-toggle--visual-end nil
+         evil-god-toggle--visual-forward nil)
+        (deactivate-mark)
+        (run-with-idle-timer
+         0 nil
+         (lambda ()
+        (evil-normal-state)))
+
+        ))
   (when (region-active-p)
     (deactivate-mark))
   (if (and evil-god-toggle--visual-beg
@@ -408,30 +494,35 @@ Restores visual selection behavior by adding `evil-visual-activate-hook' to
         (setq evil-god-toggle--visual-beg nil
               evil-god-toggle--visual-end nil
               evil-god-toggle--visual-forward nil))
-    ;; fallback: exactly like pressing `v` in normal
+    ;; fallback: exactly like pressing v in normal
     (evil-visual-char)))
 
+(defun evil-god-toggle--determine-target-state (previous-state)
+  "Determine target state based on PREVIOUS-STATE and user configuration.
+If PREVIOUS-STATE is found in `evil-god-toggle-target-state-alist`,
+return its mapped value; otherwise return PREVIOUS-STATE itself."
+  (or (alist-get previous-state evil-god-toggle-target-state-alist)
+      previous-state))
 
 (defun evil-god-toggle--exit-once ()
   "Exit `god-once` state, return to previous Evil state, or `normal' if ambiguous."
-  (let ((cmd this-command)
-        (entry-cmd 'evil-god-once-state))
-    (unless (memq cmd
-                  ;; never exit on the toggle or the state-entry commands,
-                  ;; nor on any of the built-in prefix arguments, or minibuffer
-                  (list
-                   'evil-god-toggle-once
-                   entry-cmd
-                   'universal-argument
-                   'universal-argument-minus
-                   'universal-argument-more
-                   'universal-argument-other-key
-                   'digit-argument
-                   'negative-argument))
-                   ;; also guard against being in the minibuffer:
-                   ;; `minibufferp` is a predicate, not a command, so check it separately
-      (unless (minibufferp)
-        (evil-change-to-previous-state)))))
+  (if (not (eq evil-state 'god-once))
+      (remove-hook 'post-command-hook #'evil-god-toggle--exit-once t)
+    ;; Only exit if we're not building a prefix command
+    (when (and (not (minibufferp))
+               (not (memq this-command '(evil-god-toggle-once evil-god-once-state)))
+               ;; This is the key: no prefix is being built
+               (not prefix-arg))
+      ;; Defer the state transition
+      (run-with-idle-timer
+       0 nil
+       (lambda ()
+         (let ((target-state (evil-god-toggle--determine-target-state evil-previous-state)))
+           (when (memq target-state '(god god-once god-off nil))
+             (setq target-state 'normal))
+           (evil-god-toggle-stop-god-state-maybe-visual-once target-state)
+           (evil-normalize-keymaps)))))))
+
 
 
 (provide 'evil-god-toggle)
